package me.icymint.libra.sage.core.query;

import java.sql.Connection;
import java.sql.PreparedStatement;
import java.sql.ResultSet;
import java.sql.SQLException;

import me.icymint.libra.sage.core.param.ParamHandler;
import me.icymint.libra.sage.core.result.ResultHandler;

/**
 * 查询父类。
 * 
 * @author Daniel
 * @datetime 2013-1-4上午11:13:40
 * 
 * @param <T>
 *            查询结果
 * @param <S>
 *            提供SQL语句的对象
 */
public abstract class AbstractQuery<T, S> extends
		AbstractSqlQuery<T, PreparedStatement, S, ResultSet> {

	/**
	 * 参数列表。
	 */
	private Object[] param;

	protected AbstractQuery(S sql, ResultHandler<T> rh, int maxRows,
			int[] types, Object... param) {
		super(sql, rh, maxRows, types);
		this.param = param;
	}

	/**
	 * 获取预处理语句{@link PreparedStatement}
	 */
	@Override
	protected PreparedStatement getStatement(Connection conn, String sql)
			throws SQLException {
		return conn.prepareStatement(sql);
	}

	/**
	 * 执行结果处理获取所需结果。
	 */
	@Override
	protected T handle(ResultHandler<T> rh, ResultSet v) throws SQLException {
		try {
			return rh.handle(v);
		} finally {
			v.close();
		}
	}

	/**
	 * 设置参数。
	 */
	@Override
	protected ResultSet query(PreparedStatement pps,
			ParamHandler<PreparedStatement> ph) throws SQLException {
		return ph.handle(pps, getTypes(), param).executeQuery();
	}
}
