package me.icymint.libra.sage.model;

import java.sql.SQLException;

import me.icymint.libra.sage.core.JdbcUtils;
import me.icymint.libra.sage.model.format.DefaultFormat;
import me.icymint.libra.sage.model.format.FormatSql;
import me.icymint.libra.sage.model.format.SqlFormatException;
import me.icymint.libra.sage.model.object.SqlObjectType;
import me.icymint.libra.sage.model.object.TableManagerObject;
import me.icymint.libra.sage.model.operator.SqlManager;

public class SqlFunction extends TableManagerObject implements SqlManager {

	private final SqlVar out;
	private final String sql;
	private final FormatSql format;

	protected SqlFunction(SqlInfo si, String prefix, String name,
			boolean isMask, SqlVar[] sv, SqlTable[] st, SqlVar out, String sql) {
		super(si, prefix, name, isMask, sv, st);
		this.out = out;
		format = new DefaultFormat(getJdbcUtils().getDialect(), this);
		this.sql = sql;
	}

	@Override
	public void create() throws SQLException {
		this.getJdbcUtils().execute(
				this.getJdbcUtils().getDialect().getCreateFunctionSql(this));
	}

	@Override
	public void createIfNotExist() {
		try {
			this.create();
		} catch (SQLException e) {
		}
	}

	@Override
	public void delete() throws SQLException {
		this.getJdbcUtils().execute(
				this.getJdbcUtils().getDialect().getDeleteFunctionSql(this));
	}

	public String execSql() {
		return this.getJdbcUtils().getDialect().executeFunc(this);
	}

	/**
	 * 执行自定义函数，返回结果。 请注意参数个数的匹配，否则抛出违例。
	 * 
	 * @param param
	 * @throws SQLException
	 */
	public <T, V> T execute(V... param) throws SQLException {
		if (param.length != this.getVarSize())
			throw new SQLException("参数个数不匹配！");
		// System.out.println(this.getSageUtils().getDialect().executeFunc(this));
		return this.getJdbcUtils().query(execSql(),
				JdbcUtils.getTypes(getVars()), param);
	}

	public String getFormatSql() throws SqlFormatException {
		return format.format(sql);
	}

	public SqlVar getOutVar() {
		return out;
	}

	public String getSql() {
		return sql;
	}

	@Override
	protected SqlObjectType getSqlObjectType() {
		return SqlObjectType.Function;
	}
}
