package me.icymint.libra.sage.model.format;

import java.util.regex.Matcher;
import java.util.regex.Pattern;

/**
 * 单次Pattern格式化，针对单个Pattern的格式化。
 * 
 * @author Daniel
 * @datetime 2012-11-14上午8:46:43
 * 
 */
public abstract class PatternFormat {
	private final Pattern p;

	protected PatternFormat(Pattern p) {
		this.p = p;
	}

	/**
	 * 违例信息模板。
	 * 
	 * @param sql
	 * @param source
	 * @param type
	 */
	protected String errorMessage(String sql, String source, String type) {
		return "解析字符串:" + sql + "\n" + "无法找到<" + source + ">对应的" + type + "！";
	}

	/**
	 * 格式化一组已找到的{@link Matcher#group()}的结果。
	 * <p>
	 * 该方法调用以前，已经执行了{@link Matcher#find()} ，这里需要实现从{@link Matcher#group(int)}
	 * 找到替换信息，并返回替换值。
	 * 
	 * @param sql
	 * @param m
	 * @return 返回{@link Matcher#group()}的替换值。
	 * @throws SqlFormatException
	 */
	public abstract String format(String sql, Matcher m)
			throws SqlFormatException;

	protected Pattern getPattern() {
		return p;
	}
}
