package me.icymint.libra.sage.model.operator;

import java.lang.reflect.Array;
import java.util.Arrays;

import me.icymint.libra.sage.utils.ArrayUtils;

/**
 * 封装了一个数组，并且可以对数组进行最后插入操作和删除操作。
 * 
 * @author Daniel
 * @datetime 2013-1-4下午2:16:45
 * 
 * @param <T>
 * @param <V>
 */
public abstract class ManagerArray<T, V> {
	/**
	 * 被封装的数组。
	 */
	private T[] objs;
	private final Class<T> cls;

	@SuppressWarnings("unchecked")
	public ManagerArray(T[] objs, Class<T> cls) {
		this.cls = cls;
		if (objs == null)
			objs = (T[]) Array.newInstance(cls, 0);
		this.objs = objs;
	}

	/**
	 * 添加对象。
	 * 
	 * @param t
	 * @throws SQLExistsException
	 */
	public void add(T t) throws SQLExistsException {
		try {
			get(getName(t));
			throw new SQLExistsException("相同名称的Sql对象" + getName(t) + "已经存在！");
		} catch (SQLNotExistsException e) {
			objs = ArrayUtils.concat(objs, t);
		}
	}

	public void addAll(T[] ts) {
		for (T t : ts)
			try {
				add(t);
			} catch (SQLExistsException e) {
			}
	}

	@SuppressWarnings("unchecked")
	public void clear() {
		objs = (T[]) Array.newInstance(cls, 0);
	}

	/**
	 * 删除对象。
	 * 
	 * @param t
	 */
	public void del(T t) {
		try {
			int ind = this.getIndexOf(getName(t));
			T[] vvs = Arrays.copyOf(objs, objs.length - 1);
			if (ind > 0)
				System.arraycopy(objs, 0, vvs, 0, ind - 1);
			if (ind < objs.length - 1)
				System.arraycopy(objs, ind + 1, vvs, ind, objs.length - ind - 1);
			objs = vvs;
		} catch (SQLNotExistsException e) {
		}
	}

	/**
	 * 获取ind序列对象。
	 * 
	 * @param ind
	 * @throws SQLNotExistsException
	 */
	public T get(int ind) throws SQLNotExistsException {
		try {
			return objs[ind];
		} catch (ArrayIndexOutOfBoundsException e) {
			throw new SQLNotExistsException("数组超出界限了", e);
		}
	}

	/**
	 * 获取名称为name的对象。
	 * 
	 * @param name
	 * @throws SQLNotExistsException
	 */
	public T get(V name) throws SQLNotExistsException {
		return get(getIndexOf(name));
	}

	/**
	 * 获取所有对象。
	 */
	public T[] getAll() {
		return objs;
	}

	/**
	 * 获取名称为name的对象序列号。
	 * 
	 * @param name
	 * @throws SQLNotExistsException
	 */
	public int getIndexOf(V name) throws SQLNotExistsException {
		for (int i = 0; i < objs.length; i++) {
			if (isEqual(getName(objs[i]), name))
				return i;
		}
		throw new SQLNotExistsException("未找到名称为" + name + "的Sql对象。");
	}

	protected abstract V getName(T t);

	/**
	 * 获取对象总数。
	 */
	public int getSize() {
		return objs.length;
	}

	protected abstract boolean isEqual(V v1, V v2);
}
