package me.icymint.libra.sage.model.orm;

import java.beans.PropertyEditor;
import java.beans.PropertyEditorSupport;
import java.lang.reflect.InvocationTargetException;
import java.lang.reflect.Method;
import java.util.Collection;
import java.util.LinkedHashMap;
import java.util.LinkedList;
import java.util.Map;

/**
 * 自动装箱卸箱工具。
 * 
 * @author Daniel
 * @datetime 2012-12-12上午10:08:42
 * 
 */
public class PackUtils extends PropertyEditorSupport {
	private final Map<Class<?>, PropertyEditor> map = new LinkedHashMap<Class<?>, PropertyEditor>();

	public PackUtils() {
	}

	public Object pack(Object obj, Class<?> cls) throws PackException {
		if (obj == null)
			return obj;
		if (obj instanceof String) {
			PropertyEditor pe = this.map.get(cls);
			if (pe != null) {
				pe.setAsText((String) obj);
				return pe.getValue();
			}
		} else {
			Method m = Mappings.findMethod(cls, Mappings.FETCH_BY_ID,
					obj.getClass());
			if (m != null)
				try {
					obj = m.invoke(cls, obj);
				} catch (IllegalArgumentException e) {
					throw new PackException(e);
				} catch (IllegalAccessException e) {
					throw new PackException(e);
				} catch (InvocationTargetException e) {
					throw new PackException(e);
				}

		}
		return obj;
	}

	public void register(PropertyEditor pe, Class<?> cls) {
		map.put(cls, pe);
	}

	@Override
	public void setAsText(String text)
			throws java.lang.IllegalArgumentException {
		map.clear();
		if (text.trim().length() > 0) {
			for (String cls : text.split(",")) {
				try {
					Class<?> c = Class.forName(cls);
					Object o = c.newInstance();
					if (o instanceof PropertyEditor) {
						map.put(c, (PropertyEditor) o);
					} else
						throw new java.lang.IllegalArgumentException("类" + cls
								+ "非法");
				} catch (ClassNotFoundException e) {
					throw new java.lang.IllegalArgumentException("类" + cls
							+ "没有找到", e);
				} catch (InstantiationException e) {
					throw new java.lang.IllegalArgumentException(text, e);
				} catch (IllegalAccessException e) {
					throw new java.lang.IllegalArgumentException(text, e);
				}
			}
		}
	}

	@Override
	public String toString() {
		String text = "";
		for (PropertyEditor cls : map.values()) {
			text += "," + cls.getClass().getName();
		}
		if (text.startsWith(","))
			text = text.substring(1);
		return text;
	}

	/**
	 * 单个属性的获取，卸箱。
	 * 
	 * @param obj
	 * @return
	 */
	public Object unpack(Object obj) {
		if (obj == null)
			return obj;
		PropertyEditor pe = this.map.get(obj.getClass());
		if (pe != null) {
			pe.setValue(obj);
			return pe.getAsText();
		}
		if (obj instanceof IdMapping) {
			@SuppressWarnings("unchecked")
			IdMapping<Object> xx = ((IdMapping<Object>) obj);
			return xx.fetchId();
		} else
			return obj;
	}

	public void unregister(PropertyEditor pe) {
		map.remove(pe.getValue().getClass());
	}

	public Collection<Class<?>> values() {
		Collection<Class<?>> coll = new LinkedList<Class<?>>();
		for (PropertyEditor cls : map.values()) {
			coll.add(cls.getClass());
		}
		return coll;
	}
}
